@@ -21,7 +21,7 @@ import zmq
 
 import jupyter_core.paths
 from traitlets.config import Config
-from ..notebookapp import NotebookApp
+from ..notebookapp import NotebookApp, urlencode_unix_socket
 from ..utils import url_path_join
 from ipython_genutils.tempdir import TemporaryDirectory
 
@@ -52,10 +52,10 @@ class NotebookTestBase(TestCase):
         url = cls.base_url() + 'api/contents'
         for _ in range(int(MAX_WAITTIME/POLL_INTERVAL)):
             try:
-                requests.get(url)
+                cls.fetch_url(url)
             except Exception as e:
                 if not cls.notebook_thread.is_alive():
-                    raise RuntimeError("The notebook server failed to start")
+                    raise RuntimeError("The notebook server failed to start") from e
                 time.sleep(POLL_INTERVAL)
             else:
                 return
@@ -76,6 +76,10 @@ class NotebookTestBase(TestCase):
             headers['Authorization'] = 'token %s' % cls.token
         return headers
 
+    @staticmethod
+    def fetch_url(url):
+        return requests.get(url)
+
     @classmethod
     def request(cls, verb, path, **kwargs):
         """Send a request to my server
@@ -104,7 +108,11 @@ class NotebookTestBase(TestCase):
     @classmethod
     def get_argv(cls):
         return []
-    
+
+    @classmethod
+    def get_bind_args(cls):
+        return dict(port=cls.port)
+
     @classmethod
     def setup_class(cls):
         cls.tmp_dir = TemporaryDirectory()
@@ -116,7 +124,7 @@ class NotebookTestBase(TestCase):
                 if e.errno != errno.EEXIST:
                     raise
             return path
-        
+
         cls.home_dir = tmp('home')
         data_dir = cls.data_dir = tmp('data')
         config_dir = cls.config_dir = tmp('config')
@@ -124,12 +132,15 @@ class NotebookTestBase(TestCase):
         cls.notebook_dir = tmp('notebooks')
         cls.env_patch = patch.dict('os.environ', cls.get_patch_env())
         cls.env_patch.start()
+        # Patch systemwide & user-wide data & config directories, to isolate
+        # the tests from oddities of the local setup. But leave Python env
+        # locations alone, so data files for e.g. nbconvert are accessible.
+        # If this isolation isn't sufficient, you may need to run the tests in
+        # a virtualenv or conda env.
         cls.path_patch = patch.multiple(
             jupyter_core.paths,
             SYSTEM_JUPYTER_PATH=[tmp('share', 'jupyter')],
-            ENV_JUPYTER_PATH=[tmp('env', 'share', 'jupyter')],
             SYSTEM_CONFIG_PATH=[tmp('etc', 'jupyter')],
-            ENV_CONFIG_PATH=[tmp('env', 'etc', 'jupyter')],
         )
         cls.path_patch.start()
 
@@ -140,34 +151,36 @@ class NotebookTestBase(TestCase):
 
         started = Event()
         def start_thread():
-            if 'asyncio' in sys.modules:
-                import asyncio
-                asyncio.set_event_loop(asyncio.new_event_loop())
-            app = cls.notebook = NotebookApp(
-                port=cls.port,
-                port_retries=0,
-                open_browser=False,
-                config_dir=cls.config_dir,
-                data_dir=cls.data_dir,
-                runtime_dir=cls.runtime_dir,
-                notebook_dir=cls.notebook_dir,
-                base_url=cls.url_prefix,
-                config=config,
-                allow_root=True,
-                token=cls.token,
-            )
-            # don't register signal handler during tests
-            app.init_signal = lambda : None
-            # clear log handlers and propagate to root for nose to capture it
-            # needs to be redone after initialize, which reconfigures logging
-            app.log.propagate = True
-            app.log.handlers = []
-            app.initialize(argv=cls.get_argv())
-            app.log.propagate = True
-            app.log.handlers = []
-            loop = IOLoop.current()
-            loop.add_callback(started.set)
             try:
+                bind_args = cls.get_bind_args()
+                app = cls.notebook = NotebookApp(
+                    port_retries=0,
+                    open_browser=False,
+                    config_dir=cls.config_dir,
+                    data_dir=cls.data_dir,
+                    runtime_dir=cls.runtime_dir,
+                    notebook_dir=cls.notebook_dir,
+                    base_url=cls.url_prefix,
+                    config=config,
+                    allow_root=True,
+                    token=cls.token,
+                    **bind_args
+                )
+                if 'asyncio' in sys.modules:
+                          app._init_asyncio_patch()
+                          import asyncio
+                          asyncio.set_event_loop(asyncio.new_event_loop())
+                # don't register signal handler during tests
+                app.init_signal = lambda : None
+                # clear log handlers and propagate to root for nose to capture it
+                # needs to be redone after initialize, which reconfigures logging
+                app.log.propagate = True
+                app.log.handlers = []
+                app.initialize(argv=cls.get_argv())
+                app.log.propagate = True
+                app.log.handlers = []
+                loop = IOLoop.current()
+                loop.add_callback(started.set)
                 app.start()
             finally:
                 # set the event, so failure to start doesn't cause a hang
@@ -202,6 +215,28 @@ class NotebookTestBase(TestCase):
         return 'http://localhost:%i%s' % (cls.port, cls.url_prefix)
 
 
+class UNIXSocketNotebookTestBase(NotebookTestBase):
+    # Rely on `/tmp` to avoid any Linux socket length max buffer
+    # issues. Key on PID for process-wise concurrency.
+    sock = '/tmp/.notebook.%i.sock' % os.getpid()
+
+    @classmethod
+    def get_bind_args(cls):
+        return dict(sock=cls.sock)
+
+    @classmethod
+    def base_url(cls):
+        return '%s%s' % (urlencode_unix_socket(cls.sock), cls.url_prefix)
+
+    @staticmethod
+    def fetch_url(url):
+        # Lazily import so it is not required at the module level
+        if os.name != 'nt':
+            import requests_unixsocket
+        with requests_unixsocket.monkeypatch():
+            return requests.get(url)
+
+
 @contextmanager
 def assert_http_error(status, msg=None):
     try:
